# -*- coding: utf-8 -*-


from django_rest.http.exceptions import BaseAPIException


class SerializationError(BaseAPIException):
    """ Raised when a serialization error occurs (required field that is not found, etc.) """



# -*- coding: utf-8 -*-

import json
from functools import wraps

from django.http import JsonResponse
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from django_rest.deserializers import AllPassDeserializer, Deserializer
from django_rest.http.exceptions import (
    BadRequest,
    BaseAPIException,
    InternalServerError,
    MethodNotAllowed,
    PermissionDenied,
    UnsupportedMediaType,
)
from django_rest.http.methods import SUPPORTING_PAYLOAD_METHODS
from django_rest.permissions import BasePermission

FORMS_CONTENT_TYPES = (
    "application/x-www-form-urlencoded",
    "multipart/form-data",
)


def build_deserializer_map(deserializer_class):
    if isinstance(deserializer_class, dict):
        if not all(
            issubclass(elem, Deserializer) for elem in deserializer_class.values()
        ):
            raise TypeError(
                "The values of the `deserializer_class` mapping "
                "should be subclass of `Deserializer`."
            )
        return {
            http_method: deserializer_class.get(http_method, AllPassDeserializer)
            for http_method in SUPPORTING_PAYLOAD_METHODS
        }
    if issubclass(deserializer_class, Deserializer):
        return {
            http_method: deserializer_class
            for http_method in SUPPORTING_PAYLOAD_METHODS
        }

    raise TypeError(
        "`deserializer_class` parameter should be either a subclass of `Deserializer` "
        "or a dict of `Deserializer` classes, not {}".format(
            deserializer_class.__name__
        )
    )


def transform_query_dict_into_regular_dict(query_dict):
    return {
        key: list_val if len(list_val) > 1 else list_val[0]
        for key, list_val in dict(query_dict).items()
    }


def extract_request_payload(request, allow_form_data=False):
    """
    Return the request's content (form or data).
    For `POST`, `PUT` and `PATCH` requests:
    - If the form data is allowed and the request is a form, returns a `dict`
    - If the form data isn't allowed and the request is a form, raises
      UnsupportedMediaType exception.
    - For application/json requests, returns a `dict` containing the request's
      data
    For other HTTP methods:
    - Returns `None`
    """
    method = request.method
    if not allow_form_data and request.content_type in FORMS_CONTENT_TYPES:
        raise UnsupportedMediaType
    elif request.content_type in FORMS_CONTENT_TYPES and request.method == "POST":
        return transform_query_dict_into_regular_dict(request.POST)

    if method not in SUPPORTING_PAYLOAD_METHODS:
        return None

    return json.loads(request.body.decode())


def build_function_wrapper(
    permission_class,  # type: BasePermission
    allowed_methods,  # type: Iterable[str]
    deserializers_http_methods_map,  # type: Dict[str, ClassVar[Deserializer]]
    allow_forms,  # type: bool
    view_function,  # Callable
):  # type:(...) -> Callable
    @wraps(view_function)
    def function_wrapper(request, *args, **kwargs):
        # type:(HttpRequest, List[Any]) -> JsonResponse
        try:
            if request.method not in allowed_methods:
                raise MethodNotAllowed
            if not permission_class().has_permission(request, view_function):
                raise PermissionDenied
            query_params = transform_query_dict_into_regular_dict(request.GET)
            payload = extract_request_payload(request, allow_forms)
            if request.method in SUPPORTING_PAYLOAD_METHODS:
                deserializer = deserializers_http_methods_map[request.method](
                    data=payload
                )
                if not deserializer.is_valid():
                    raise BadRequest
                deserialized_data = deserializer.data
            else:
                deserialized_data = None

            return view_function(
                request,
                url_params=kwargs,
                query_params=query_params,
                deserialized_data=deserialized_data,
            )
        except BaseAPIException as e:
            return JsonResponse({"error_msg": e.RESPONSE_MESSAGE}, status=e.STATUS_CODE)
        except Exception:
            return JsonResponse(
                {"error_msg": InternalServerError.RESPONSE_MESSAGE},
                status=InternalServerError.STATUS_CODE,
            )

    return csrf_exempt(function_wrapper)


def build_class_wrapper(
    permission_class,  # type: BasePermission
    allowed_methods,  # type: Iterable[str]
    deserializers_http_methods_map,  # type: Dict[str, ClassVar[Deserializer]]
    allow_forms,  # type: bool
    view_class,  # type: ClassVar
):  # type:(...) -> ClassVar
    class ViewWrapper(View):
        __doc__ = view_class.__doc__
        __module__ = view_class.__module__
        __slots__ = "_wrapped_view"

        def __init__(self, *args, **kwargs):
            self._wrapped_view = view_class(*args, **kwargs)
            self.compiled_dispatch = build_function_wrapper(
                permission_class,
                allowed_methods,
                deserializers_http_methods_map,
                allow_forms,
                self._handle_request,
            )

        def _handle_request(self, request, *args, **kwargs):
            # type:(HttpRequest, List[Any]) -> JsonResponse
            # No need for additional check on request.method, since it's been
            # already checked
            handler = getattr(self, request.method.lower())
            return handler(request, *args, **kwargs)

        @csrf_exempt
        def dispatch(self, *args, **kwargs):
            # type:(HttpRequest, List[Any]) -> JsonResponse
            return self.compiled_dispatch(*args, **kwargs)

        def __getattribute__(self, name):
            try:
                return super(ViewWrapper, self).__getattribute__(name)
            except AttributeError:
                attribute = self._wrapped_view.__getattribute__(name)
                return attribute

    ViewWrapper.__name__ = view_class.__name__
    return ViewWrapper



# -*- coding: utf-8 -*-

import inspect

from django.views import View

from django_rest.decorators.utils import (
    build_class_wrapper,
    build_deserializer_map,
    build_function_wrapper,
)
from django_rest.deserializers import AllPassDeserializer, Deserializer
from django_rest.http.methods import ALL_METHODS
from django_rest.permissions import AllowAny, BasePermission

FORMS_CONTENT_TYPES = (
    "application/x-www-form-urlencoded",
    "multipart/form-data",
)


def api_view(
    permission_class=AllowAny,  # type: Union[AbstractPermission, Any]
    allowed_methods=ALL_METHODS,  # type: Tuple[str]
    deserializer_class=AllPassDeserializer,  # type: Union[ClassVar[Deserializer], Dict[str, ClassVar[Deserializer]]]
    allow_forms=False,  # type: bool
):  # type:(...) -> Callable

    if not (
        inspect.isclass(permission_class)
        and issubclass(permission_class, BasePermission)
    ):
        # In case the decorator's called without parenthesis (`@api_view`),
        # the `permission_class` variable holds the real view.
        return api_view()(permission_class)

    deserializers_http_methods_map = build_deserializer_map(deserializer_class)

    def view_decorator(view):
        # type:(Union[Callable, ClassVar]) -> Union[Callable, ClassVar]
        if inspect.isfunction(view):
            return build_function_wrapper(
                permission_class,
                allowed_methods,
                deserializers_http_methods_map,
                allow_forms,
                view,
            )
        elif inspect.isclass(view) and issubclass(view, View):
            return build_class_wrapper(
                permission_class,
                allowed_methods,
                deserializers_http_methods_map,
                allow_forms,
                view,
            )
        given_str = (
            "{} class".format(view.__name__)
            if inspect.isclass(view)
            else "{} object".format(view.__class__.__name__)
        )
        raise TypeError(
            "The `@api_view` decorator is applied to either a function or a "
            "class inheriting from django.views.View. Given: {}".format(given_str)
        )

    return view_decorator



# -*- coding: utf-8 -*-

import types


class Field(object):
    """The Field class is used to define what attributes will be serialized.

    It maps a property or function on an object to a value in the serialized result.
    Subclass this to make custom fields. For most simple cases, overriding
    Field.to_value() should be enough. For more control, you may override
    Field.as_getter().

    :param str attr_name: The attribute to get on the object. If not supplied,
        the name the current field was assigned to inside the serializer will be used.
    :param bool call: Whether the value should be called after it is retrieved
        from the object. Useful if an object has a method to be serialized.
    :param str label: A label to use as the name of the serialized field
        instead of using the attribute name of the field.
    :param bool required: Whether the field is required. If set to `False`,
        the method Field.to_value() will not be called if the value is `None` or
        an error is raised during its serialization.
    """

    __slots__ = ("attr_name", "call", "label", "required")

    #  Set to `True` if the value function returned from the method Field.as_getter()
    #  requires the serializer to be passed in as the first argument. Otherwise,
    #  the object will be the only parameter.
    getter_needs_serializer_as_arg = False

    def __init__(self, attr_name=None, call=False, label=None, required=True):
        self.attr_name = attr_name
        self.call = call
        self.label = label
        self.required = required

    def to_value(self, value):
        # type:(Any) -> Any
        """Transforms the serialized value. It could be used for cleaning
        and validating the value serialized by the current field. For example,
        to implement an `int` field, the Field.to_value() method will looks like:

            def to_value(self, value):
                return int(value)

        :param value: The value fetched from the object being serialized.
        """
        return value

    to_value._base_implementation = True

    def _is_to_value_overridden(self):
        # type:() -> bool
        to_value = self.to_value
        # If to_value isn't a method, it must have been overridden.
        if not isinstance(to_value, types.MethodType):
            return True
        return not getattr(to_value, "_base_implementation", False)

    def as_getter(self, serializer_field_name, serializer_cls):
        # type:(str, type) -> Optional[Callable]
        """Returns a function that fetches an attribute from an object.

        If `None` is returned, the default getter defined in `Serializer.default_getter`
        will be used instead.

        When a `Serializer` class is defined, each `Field` class will be
        compiled into a getter function using `æs_geter()` method. During the serialization
        process, each getter will be called with the object being serialized, then,
        the value returned from the getter will be passed through Field.to_value()
        method.

        If a `Field` class has set the `getter_needs_serializer_as_arg` to `True`,
        then the getter returned from the current method will be called with the
        `Serializer` instance as the first argument, and the object being serialized
        as the second.

        :param str serializer_field_name: The name this field was assigned to on the
            serializer.
        :param serializer_cls: The :class:`Serializer` this field is defined in.
        """
        return None


class SerializerBase(Field):
    _field_map = {}



# -*- coding: utf-8 -*-

import types

import six

from django_rest.serializers.serializers import Serializer
from django_rest.serializers.base import Field
from django_rest.serializers.exceptions import SerializationError

if six.PY34:
    from collections.abc import Iterable, Mapping  # pragma: no cover
else:
    from collections import Iterable, Mapping  # pragma: no cover


NoneType = type(None)
UnicodeType = types.UnicodeType if six.PY2 else str


PRIMITIVE_TYPES = (
    bool,
    int,
    float,
    str,
    NoneType,
    UnicodeType,
)
ITERABLE_TYPES = (
    Iterable,
    Mapping,
)
ALLOWED_CONSTANT_TYPES = ITERABLE_TYPES + PRIMITIVE_TYPES


class CharField(Field):
    """ Field that converts the attribute's value into a string. """

    to_value = staticmethod(six.text_type)


class IntegerField(Field):
    """ Field that converts the attribute's value into an integer. """

    to_value = staticmethod(int)


class FloatField(Field):
    """ Field that converts the attribute's value into a float. """

    # TODO: implement precision

    to_value = staticmethod(float)


class BooleanField(Field):
    """ Field that converts the attribute's value into a boolean. """

    to_value = staticmethod(bool)


def _is_valid_field_instance(field_instance):
    # type:(Field) -> None
    assert not isinstance(
        field_instance, Serializer
    ), "Cannot call `ListField()` with a Serializer object. An option `many=True` is available for serializers."

    assert (
        field_instance._is_to_value_overridden()
    ), "`ListField()` can only be called with primitive-types fields. Given field type: {}".format(
        field_instance.__class__.__name__
    )


def ListField(field_instance):
    # type:(Field) -> Field
    """Allows to apply the Field.to_value() method on an iterable of values,
    instead of a single value. The same purpose could be achieved with `MethodField()`,
    but it'll be just too annoying.
    Example:

        class Subscription:
            status = ("active", "trialing", "canceled")

        class SubscriptionSerializer(Serializer):
            status = ListField(CharField(label="allowed_status", required=True))

        serialized_obj = SubscriptionSerializer(subscription).data
        # serialized_obj = {"allowed_status": ["active", "trialing", "canceled"]}

    :param field_instance: a Field instance that represents the type of a single element
        of the iterable to be passed to `to_value()` method.
        Note that the `field_instance` cannot be a `MethodField()`, a `ConstantField()`
        or a `Serializer` (which has the option `many=True` for the same purpose).
    The `ListField` generates a new field class (and instance) during the declaration
    (before the runtime), for perfomance purpose.
    """
    _is_valid_field_instance(field_instance)
    field_class = field_instance.__class__
    ListFieldClass = type(
        "List{}".format(field_class.__name__),
        (field_class,),
        {"__doc__": field_class.__doc__},
    )
    attrs = {name: getattr(field_instance, name) for name in field_instance.__slots__}

    def to_value(value):
        return [field_instance.to_value(v) for v in value]

    ListFieldClass.to_value = staticmethod(to_value)
    return ListFieldClass(**attrs)


class ConstantField(Field):
    """
    A Field that allows to constant data injection into the serialized object, without
    the need to declare an (serialized) object method, or a serializer's `MethodField()`
    to return a constant.
    For example:

        foo_const = "Foo"
        class FooSerializer(Serializer):
            foo = ConstantField(label="foo_constant", required=False, constant=foo_const)
            bar = IntegerField()
        foo = Foo(bar=5)
        serialized_foo = FooSerializer(foo).data
        # serialized_foo = {'foo': "Foo", 'bar': 5}


    :param str label: Same as other fields.
    :param bool required: Same as other fields.
    :param constant: The constant to be added in the serialized object. Should be
        a combination (`list`, `dict` or single value) of primitive type (int, float,
        str, bool, None).
    """

    __slots__ = ("label", "required", "constant", "_is_primitive")

    def __init__(self, constant=None, label=None, required=True):
        self.label = label
        self.required = required
        self.attr_name = None
        self.call = False
        self._is_primitive = self._is_primitive_const(constant)

        if not self._is_primitive and self.required:
            raise SerializationError(
                "Only primitive types are accepted in `ConstantField` (int, float, str, "
                "bool, None, unicode) and iterables/dict of primitive types. "
                "type({}) = {}.".format(constant, constant.__class__.__name__)
            )
        self.constant = constant

    def as_getter(self, serializer_field_name, serializer_cls):
        # type:(str, Serializer) -> Callable
        constant = self.constant

        def getter(obj):
            if not self._is_primitive and not self.required:
                # This raise will make the `_serialize` method `continue` the
                # loop without saving the current field
                raise TypeError(
                    "Non-required field: Non primitive constant given: {}".format(
                        constant.__class__.__name__
                    )
                )
            return constant

        return getter

    @classmethod
    def _is_primitive_const(cls, constant):
        # type:(Any) -> bool
        if not isinstance(constant, ALLOWED_CONSTANT_TYPES):
            return False

        if isinstance(constant, PRIMITIVE_TYPES):
            return True

        if isinstance(constant, Mapping):
            keys_are_primitive = all(
                isinstance(key, (str, UnicodeType)) for key in constant.keys()
            )
            return keys_are_primitive and all(
                cls._is_primitive_const(value) for value in constant.values()
            )

        # The case of iterable
        return all(cls._is_primitive_const(element) for element in constant)


class MethodField(Field):
    """A Field class that calls a method on the `Serializer` class.

    This is useful if a Field needs to serialize a value that may come from multiple
    attributes on the same object. For example:

        class FooSerializer(Serializer):
            plus = MethodField()
            minus = MethodField('do_minus')

            def get_plus(self, foo_obj):
                return foo_obj.bar + foo_obj.baz

            def do_minus(self, foo_obj):
                return foo_obj.bar - foo_obj.baz

        foo = Foo(bar=5, baz=10)
        serialized_obj = FooSerializer(foo).data
        # serialized_obj = {'plus': 15, 'minus': -5}

    :param str method_name: The serializer's method name to be called. Defaults to
        `get_<field name>`.
    """

    __slots__ = ("label", "required", "method_name")
    getter_needs_serializer_as_arg = True

    def __init__(self, method_name=None, **kwargs):
        super(MethodField, self).__init__(**kwargs)
        self.method_name = method_name

    def as_getter(self, serializer_field_name, serializer_cls):
        # type:(str, Serializer) -> Callable
        method_name = self.method_name
        if method_name is None:
            method_name = "get_{0}".format(serializer_field_name)
        return getattr(serializer_cls, method_name)



# -*- coding: utf-8 -*-

from django_rest.serializers import fields
from django_rest.serializers.serializers import (
    DictSerializer,
    Field,
    Serializer,
    SerializationError,
)

__all__ = ["fields", "DictSerializer", "Field", "Serializer", "SerializationError"]



# -*- coding: utf-8 -*-

from django.forms import fields

from django_rest.deserializers.base import (
    AllPassDeserializer,
    Deserializer,
    ValidationError,
)

__all__ = ["fields", "AllPassDeserializer", "Deserializer", "ValidationError"]



# -*- coding: utf-8 -*-

import operator

import six

from django_rest.serializers.base import Field, SerializerBase
from django_rest.serializers.exceptions import SerializationError


def _compile_field_to_tuple(field, serializer_field_name, serializer_cls):
    getter = field.as_getter(serializer_field_name, serializer_cls)
    if getter is None:
        getter = serializer_cls.default_getter(field.attr_name or serializer_field_name)

    # Only set a to_value function if it has been overridden for performance.
    to_value = None
    if field._is_to_value_overridden():
        to_value = field.to_value

    # Set the field name to a supplied label; defaults to the attribute name.
    name = field.label or serializer_field_name

    return (
        name,
        getter,
        to_value,
        field.call,
        field.required,
        field.getter_needs_serializer_as_arg,
    )


class SerializerMeta(type):
    @staticmethod
    def _get_fields(direct_fields, serializer_cls):
        field_map = {}
        # Get all the fields from base classes.
        for cls in serializer_cls.__mro__[::-1]:
            if issubclass(cls, SerializerBase):
                field_map.update(cls._field_map)
        field_map.update(direct_fields)
        return field_map

    @staticmethod
    def _compile_fields(field_map, serializer_cls):
        return [
            _compile_field_to_tuple(field, name, serializer_cls)
            for name, field in field_map.items()
        ]

    def __new__(cls, name, bases, attrs):
        # Fields declared directly on the class.
        direct_fields = {}

        # Take all the Fields from the attributes.
        for attr_name, field in attrs.items():
            if isinstance(field, Field):
                direct_fields[attr_name] = field
        for k in direct_fields.keys():
            del attrs[k]

        real_cls = super(SerializerMeta, cls).__new__(cls, name, bases, attrs)

        field_map = cls._get_fields(direct_fields, real_cls)
        compiled_fields = cls._compile_fields(field_map, real_cls)

        real_cls._field_map = field_map
        real_cls._compiled_fields = tuple(compiled_fields)
        return real_cls


class Serializer(six.with_metaclass(SerializerMeta, SerializerBase)):
    """The Serializer class is used as a base for custom serializers.

    A Serializer class is also a subclass of Field class, which allows nesting
    Serializers. A new serializer is defined by subclassing the `Serializer` class,
    then adding each `Field` as a class variable.

    Example: :

        class FooSerializer(Serializer):
            foo = Field()
            bar = Field()

        foo = Foo(foo='hello', bar=5)
        serialized_obj = FooSerializer(foo).data
        # serialized_obj = {'foo': 'hello', 'bar': 5}

    :param instance: The object or iterable of objects to serialize.
    :param bool many: If `instance` is an iterable of objects, set `many` to `True`
        to serialize it as a list.
    """

    #: The default getter used if `Field.as_getter()` returns None.
    default_getter = operator.attrgetter

    def __init__(self, instance=None, many=False, **kwargs):
        super(Serializer, self).__init__(**kwargs)
        self.instance = instance
        self.many = many
        self._data = None

    def _serialize(self, instance, fields):
        # Transforms the `instance` object into its serialized value, by using
        # the `fields` list parameter.
        serialized_value = {}
        for name, getter, to_value, call, required, pass_self in fields:
            try:
                if pass_self:
                    result = getter(self, instance)
                else:
                    result = getter(instance)
                    if required or result is not None:
                        if call:
                            result = result()
                        if to_value:
                            result = to_value(result)
            except (KeyError, AttributeError, TypeError, ValueError) as e:
                if required:
                    raise SerializationError(str(e))
            else:
                serialized_value[name] = result

        return serialized_value

    def to_value(self, instance):
        fields = self._compiled_fields
        if self.many:
            serialize = self._serialize
            return [serialize(o, fields) for o in instance]
        return self._serialize(instance, fields)

    @property
    def data(self):
        # type:() -> Dict[str, Any]
        """Get the serialized data from the Serializer instance. The data is cached
        for further accesses.
        """
        # Cache the data for next time `.data` is called.
        if self._data is None:
            self._data = self.to_value(self.instance)
        return self._data


class DictSerializer(Serializer):
    """DictSerializer serializes python `dicts` instead of objects.

    `DictSerializer` uses `operator.itemgetter` to fetch data from the object
    to serialize, while `Serializer` class uses `operator.attrgetter`.

    Example: ::

        class FooSerializer(DictSerializer):
            foo = IntField()
            bar = FloatField()

        foo = {'foo': '5', 'bar': '2.2'}
        serialized_obj = FooSerializer(foo).data
        # serialized_obj = {'foo': 5, 'bar': 2.2}
    """

    default_getter = operator.itemgetter



# -*- coding: utf-8 -*-

import six
from django.core.exceptions import NON_FIELD_ERRORS, ValidationError
from django.forms.fields import Field
from django.forms.forms import DeclarativeFieldsMetaclass
from django.forms.utils import ErrorDict, ErrorList


class BaseDeserializer(Field):
    """
    The main implementation of Deserializers logic. Note that this class is
    slightly different from the classic Form.
    """

    __slots__ = ()

    empty_values = (None, "")

    def __init__(self, data=None, **kwargs):
        super(BaseDeserializer, self).__init__(**kwargs)
        self._init_data(data)

    def _init_data(self, data):
        # type:(Optional[Dict]) -> None
        self._data = data
        self._errors = None  # Stores the errors after `full_clean()` has been called.
        self._cleaned_data = None

    @classmethod
    def fields(cls):
        # type:() -> Iterator[str, Field]
        for name in cls.declared_fields:
            yield name, cls.declared_fields[name]

    @property
    def errors(self):
        # type:() -> ErrorDict
        """Return an ErrorDict for the data provided for the deserializer."""
        if self._errors is None:
            self.full_clean()
        return self._errors

    def is_valid(self):
        """Return True if the deserializer has no errors, or False otherwise."""
        return not self.errors

    def _get_error_dict(self, field_name, error):
        # type:(str, ValidationError) -> Dict
        if not hasattr(error, "error_dict"):
            # In case the error comes from simple ("flat") Field
            return {field_name or NON_FIELD_ERRORS: error.messages}

        # Nested field case (the Field itself is a Deserializer instance)
        return {field_name: dict(error)}

    def add_error(self, field_name, error):
        # type:(str, ValidationError) -> None
        """
        Updates the content of `self._errors`. Each `self._errors` is a `dict` instance.
        """
        error = self._get_error_dict(field_name, error)
        for field_name, errors in error.items():
            self._errors[field_name] = errors

    def full_clean(self):
        """
        Clean all of self._data and populate self._errors and self._cleaned_data.
        """
        self._errors = ErrorDict()
        self._cleaned_data = {}
        self._clean_fields()

    def _clean_fields(self):
        if self._data in self.empty_values:
            self._cleaned_data = self._data
            return
        if not isinstance(self._data, dict):
            error_list = ErrorList(["This field should be an object."])
            raise ValidationError(error_list)
        for name, field in self.declared_fields.items():
            value = self._data.get(name)
            try:
                value = field.clean(value)
                self._cleaned_data[name] = value
                if hasattr(self, "post_clean_{}".format(name)):
                    value = getattr(self, "post_clean_{}".format(name))(value)
                    self._cleaned_data[name] = value
            except ValidationError as e:
                if field.required:
                    self.add_error(name, e)

    def clean(self, data):
        """
        Overriden version of superclass Field.clean(). Called in case of nested
        Deserializers. Performs the validation and cleaning process on the data
        parameter. Returns the cleaned data to the parent deserializer.
        In case of error, Raises `ValidationError` with the whole ErrorDict()
        of its errors in order to be catched in the parent.
        """

        # Raises an error if `data` is empty and the current (nested)
        # deserializer is required.
        self.validate(data)

        self._init_data(data)
        if self.errors:
            raise ValidationError(self.errors)
        return self._cleaned_data

    @property
    def data(self):
        # type:() -> dict
        if not self._cleaned_data:
            self.full_clean()
        return self._cleaned_data


class Deserializer(six.with_metaclass(DeclarativeFieldsMetaclass, BaseDeserializer)):
    pass


class AllPassDeserializer(Deserializer):
    """Deserializer that accepts any given data. the clean always returns
    the data received as input.
    """

    def _clean_fields(self):
        for name, value in self._data.items():
            self._cleaned_data[name] = value



from io import open
from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="django-rest",
    version="0.8.7",
    url="https://github.com/alaouimehdi1995/django-rest/",
    license="MIT",
    description="Tiny, lightweight, blazing fast REST library for django",
    keywords="django REST library fast light serializer",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Mehdi Alaoui",
    author_email="alaoui.mehdi.1995@gmail.com",
    packages=find_packages(include=["django_rest*"]),
    install_requires=["django>=1.11", "six"],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, <4",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 1.4",
        "Framework :: Django :: 1.5",
        "Framework :: Django :: 1.6",
        "Framework :: Django :: 1.7",
        "Framework :: Django :: 1.8",
        "Framework :: Django :: 1.9",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
    ],
    project_urls={"Source": "https://github.com/alaouimehdi1995/django-rest/"},
)



# -*- coding: utf-8 -*-

import six

from django_rest.http.methods import SAFE_METHODS


class MetaOperand(type):
    """Metaclass that allows its instances (permission classes) to use logical
    using logical operators (AND, OR, ..) with the follwing syntax:

        FinalPermClass = (Perm1 | Perm2) & ~Perm3

    The `OPERATOR_NAME` class attribute is used to build a name for the resulting
    class of an unary/binary operation. For example:

        FinalPermClass = (Perm1 | Perm2) & ~Perm3
        FinalPermClass.__class__.__name__
        # ((Perm1_OR_Perm2)_And_(Not_Perm3))

    """

    OPERATOR_NAME = None

    def __and__(first_class, second_class):
        # type:(ClassVar, ClassVar) -> ClassVar
        return AND.build_permission_from(first_class, second_class)

    def __or__(first_class, second_class):
        # type:(ClassVar, ClassVar) -> ClassVar
        return OR.build_permission_from(first_class, second_class)

    def __xor__(first_class, second_class):
        # type:(ClassVar, ClassVar) -> ClassVar
        return XOR.build_permission_from(first_class, second_class)

    def __invert__(first_class):
        # type:(ClassVar, ClassVar) -> ClassVar
        return NOT.build_permission_from(first_class)


# == Abstract operators


class BinaryOperator(object):
    """Class that describes how to build a permission class as a result of a
    Binary operators only. The current class is intended to be inherited by operators
    like: `AND`, `OR`, `XOR`, etc.

    The method called during operations is `build_permission_class()`. It creates
    a new permission class (i.e. inheriting from `BasePermission`), for which
    the `has_permission()` method is built using the `calculate()` method on both
    `has_permission()` results of operand classes.

    A new binary operator can be created by inheriting both the current class, and
    defining `MetaOperand` as metaclass, then implementing the `calculate()`
    staticmethod.
    Example:

        class MyNewBinaryOperator(BinaryOperator, metaclass=MetaOperand):
            @staticmethod
            def calculae(value1, value2):
                # type:(bool, bool) -> bool
                return fn(value1, value2)

    Then, in order to make the new operator used (without having to always call
    `MyNewBinaryOperator.build_permission_from(class1, class2)`, it should be
    subscribed in the `MetaOperand`. For example, in order to use the new operator
    with the `+`, it should be assigned in the `__add__` method of `MetaOperand`
    """

    @staticmethod
    def calculate(first_function, second_function, *args, **kwargs):
        # type:(Callable, Callable, *Any, **Any) -> bool
        raise NotImplementedError(
            "`calculate()` method should be defined in subclasses"
        )

    @classmethod
    def build_classname(cls, class_1, class_2):
        # type:(ClassVar, ClassVar) -> str
        return "({}{}{})".format(class_1.__name__, cls.OPERATOR_NAME, class_2.__name__)

    @classmethod
    def build_docstring(cls, class_1, class_2):
        # type:(ClassVar, ClassVar) -> str
        return "\t({})\n\t{}\n\t({})".format(
            class_1.__doc__, cls.OPERATOR_NAME, class_2.__doc__
        )

    @classmethod
    def build_permission_from(cls, permission_class_1, permission_class_2):
        # type:(ClassVar, ClassVar) -> ClassVar
        result_classname = cls.build_classname(permission_class_1, permission_class_2)
        result_class = MetaOperand(result_classname, (BasePermission,), {})
        result_class.__doc__ = cls.build_docstring(
            permission_class_1, permission_class_2
        )

        def has_permission(self, request, view):
            # type:(HttpRequest, Callable) -> bool
            permission_func_1 = permission_class_1().has_permission
            permission_func_2 = permission_class_2().has_permission
            return cls.calculate(permission_func_1, permission_func_2, request, view)

        result_class.has_permission = has_permission
        return result_class


class UnaryOperator(object):
    """Class that describes how to build a permission class as a result of a
    Unary operators only. The current class is intended to be inherited by operators
    like: `NOT` and Identity operators.

    The method called during operations is `build_permission_class()`. It creates
    a new permission class (i.e. inheriting from `BasePermission`), for which
    the `has_permission()` method is built using the `calculate()` method on the
    `has_permission()` result of operand class.

    A new unary operator can be created by inheriting both the current class, and
    defining `MetaOperand` as metaclass, then implementing the `calculate()`
    staticmethod.
    Example:

        class MyNewUnaryOperator(UnaryOperator, metaclass=MetaOperand):
            @staticmethod
            def calculae(value):
                # type:(bool) -> bool
                return fn(value)
    """

    OPERATOR_NAME = None

    @staticmethod
    def calculate(function, *args, **kwargs):
        # type:(Callable, *Any, **Any) -> bool
        raise NotImplementedError(
            "`calculate()` method should be defined in subclasses"
        )

    @classmethod
    def build_classname(cls, _class):
        # type:(ClassVar) -> str
        return "({}{})".format(cls.OPERATOR_NAME, _class.__name__)

    @classmethod
    def build_docstring(cls, _class):
        # type:(ClassVar, ClassVar) -> str
        return "\t{}({})".format(cls.OPERATOR_NAME, _class.__doc__)

    @classmethod
    def build_permission_from(cls, permission_class):
        # type:(ClassVar) -> ClassVar
        result_classname = cls.build_classname(permission_class)
        result_class = MetaOperand(result_classname, (BasePermission,), {})
        result_class.__doc__ = cls.build_docstring(permission_class)

        def has_permission(self, request, view):
            # type:(HttpRequest, Callable) -> bool
            permission_func = permission_class().has_permission
            return cls.calculate(permission_func, request, view)

        result_class.has_permission = has_permission
        return result_class


# == Concrete operators


class AND(six.with_metaclass(MetaOperand, BinaryOperator)):
    """AND Logical operator class.

    Example of use:

        ResultPermClass = AND.build_permission_from(PermClass1, PermClass2)
        # Which is also equivalend to:
        ResultPermClass = PermClass1 & PermClass2
    """

    OPERATOR_NAME = "_AND_"

    @staticmethod
    def calculate(first_function, second_function, *args, **kwargs):
        # type:(Callable, Callable, *Any, **Any) -> bool
        return first_function(*args, **kwargs) and second_function(*args, **kwargs)


class OR(six.with_metaclass(MetaOperand, BinaryOperator)):
    """OR Logical operator class.

    Example of use:

        ResultPermClass = OR.build_permission_from(PermClass1, PermClass2)
        # Which is also equivalend to:
        ResultPermClass = PermClass1 | PermClass2
    """

    OPERATOR_NAME = "_OR_"

    @staticmethod
    def calculate(first_function, second_function, *args, **kwargs):
        # type:(Callable, Callable, *Any, **Any) -> bool
        return first_function(*args, **kwargs) or second_function(*args, **kwargs)


class XOR(six.with_metaclass(MetaOperand, BinaryOperator)):
    """XOR (eXclusive OR) Logical operator class.

    Example of use:

        ResultPermClass = XOR.build_permission_from(PermClass1, PermClass2)
        # Which is also equivalend to:
        ResultPermClass = PermClass1 ^ PermClass2
    """

    OPERATOR_NAME = "_XOR_"

    @staticmethod
    def calculate(first_function, second_function, *args, **kwargs):
        # type:(Callable, Callable, *Any, **Any) -> bool
        return first_function(*args, **kwargs) ^ second_function(*args, **kwargs)


class NOT(six.with_metaclass(MetaOperand, UnaryOperator)):
    """NOT Logical operator class.

    Example of use:

        ResultPermClass = NOT.build_permission_from(PermClass)
        # Which is also equivalend to:
        ResultPermClass = ~PermClass
    """

    OPERATOR_NAME = "NOT_"

    @staticmethod
    def calculate(function, *args, **kwargs):
        # type:(Callable, *Any, **Any) -> bool
        return not function(*args, **kwargs)


class BasePermission(six.with_metaclass(MetaOperand, object)):
    """
    The Mainclass of all existing permissions. It's created with `MetaOperand`,
    which allows it to use operators such as: `&`, `|` `^` and `~`.

    A new custom permission could be created by inheriting from the current class,
    and implementing the `has_permission()` method, as shown in the following example:

        class FooBarPermission(BasePermission):
            def has_permission(self, request, view):
                return request.user.name in ("foo", "bar")
    """

    def has_permission(self, request, view):
        # type:(HttpRequest, Callable) -> bool
        raise NotImplementedError(
            "`has_permission()` method should be defined in subclasses"
        )


class AllowAny(BasePermission):
    """
    Allows everybody to access the view.
    """

    def has_permission(self, request, view):
        # type:(HttpRequest, Callable) -> bool
        return True


class IsAuthenticated(BasePermission):
    """
    Allows the view access to authenticated users only.
    """

    def has_permission(self, request, view):
        # type:(HttpRequest, Callable) -> bool
        return bool(request.user and request.user.is_authenticated)


class IsStaffUser(BasePermission):
    """
    Allows the view access to staff users only.
    """

    def has_permission(self, request, view):
        # type:(HttpRequest, Callable) -> bool
        return bool(request.user and request.user.is_staff)


class IsAdminUser(BasePermission):
    """
    Allows the view access to admin users only.
    """

    def has_permission(self, request, view):
        # type:(HttpRequest, Callable) -> bool
        return bool(request.user and request.user.is_superuser)


class IsReadOnly(BasePermission):
    """
    Allows the view access to read-only http methods: GET, HEAD and OPTIONS.
    """

    def has_permission(self, request, view):
        # type:(HttpRequest, Callable) -> bool
        return request.method in SAFE_METHODS


IsAuthenticatedOrReadOnly = IsAuthenticated | IsReadOnly



# -*- coding: utf-8 -*-

from django_rest.http import status


class BaseAPIException(Exception):
    """
    Base API exception from which all specific exceptions will inherit.
    The exception defines a `STATUS_CODE` attribute and a `RESPONSE_MESSAGE`
    Those attributes are used by `api_view()` decorator, to return a readable
    `JsonResponse()` with a valid message and status code, without having to catch
    exceptions inside the view.
    """

    STATUS_CODE = status.HTTP_500_INTERNAL_SERVER_ERROR
    RESPONSE_MESSAGE = "An unknown server error occured."


class BadRequest(BaseAPIException):
    STATUS_CODE = status.HTTP_400_BAD_REQUEST
    RESPONSE_MESSAGE = "Bad request."


class NotAuthenticated(BaseAPIException):
    STATUS_CODE = status.HTTP_401_UNAUTHORIZED
    RESPONSE_MESSAGE = "Unauthorized operation. Maybe forgot the authentication step ?"


class PermissionDenied(BaseAPIException):
    STATUS_CODE = status.HTTP_403_FORBIDDEN
    RESPONSE_MESSAGE = "Forbidden operation. Make sure you have the right permissions."


class NotFound(BaseAPIException):
    STATUS_CODE = status.HTTP_404_NOT_FOUND
    RESPONSE_MESSAGE = "The requested resource is not found."


class MethodNotAllowed(BaseAPIException):
    STATUS_CODE = status.HTTP_405_METHOD_NOT_ALLOWED
    RESPONSE_MESSAGE = "HTTP Method not allowed."


class UnsupportedMediaType(BaseAPIException):
    STATUS_CODE = status.HTTP_415_UNSUPPORTED_MEDIA_TYPE
    RESPONSE_MESSAGE = "Unsupported Media Type. Check your request's Content-Type."


class InternalServerError(BaseAPIException):
    pass


class ServiceUnavailable(BaseAPIException):
    STATUS_CODE = status.HTTP_503_SERVICE_UNAVAILABLE
    RESPONSE_MESSAGE = "The requested service is unavailable."



# -*- coding: utf-8 -*-

"""
For more details: https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods
"""


# Read-only methods
HEAD = "HEAD"
GET = "GET"

# Writing methods
POST = "POST"
PUT = "PUT"
PATCH = "PATCH"
DELETE = "DELETE"

# Special methods
OPTIONS = "OPTIONS"
TRACE = "TRACE"
CONNECT = "CONNECT"

SAFE_METHODS = (GET, HEAD, OPTIONS)

SUPPORTING_PAYLOAD_METHODS = (
    POST,
    PUT,
    PATCH,
)

ALL_METHODS = (
    HEAD,
    GET,
    POST,
    PUT,
    PATCH,
    DELETE,
    OPTIONS,
    TRACE,
    CONNECT,
)
